<?php
/**
 * @filename StringReader.php
 * @author Jerry Yan <792602257@qq.com>
 * @date 2020/12/17 14:58
 */


namespace JerryYan\DSL\Reader;


class StringReader extends ReaderInterface
{
    protected $string;

    public function __construct(string $string)
    {
        $this->string = $string;
        $this->moveToNextToken();
    }

    /**
     * @inheritDoc
     */
    public function getNextChar(int $startAt = null): string
    {
        if ($startAt === null) $startAt = $this->currentPosition;
        if (mb_strlen($this->string) <= $startAt) return "";
        return mb_substr($this->string, $startAt, 1);
    }

    /**
     * @inheritDoc
     */
    public function getNextToken(): string
    {
        $curToken = "";
        $curPos = $this->nextPosition;
        while (mb_strlen($curChar = $this->getNextChar($curPos)) > 0) {
            $curPos++;
            switch ($curChar) {
                case " ":
                    // 如果开始的时候就有空白，跳过它
                    if (empty($curToken)) {
                        continue 2;
                    }
                    // 否则就结束（已经匹配完成）
                    break 2;
                case "\r":
                case "\n":
                    if (empty($curToken)) {
                        continue 2;
                    }
                    break 2;
                default:
                    $curToken .= $curChar;
            }
        }
        return $curToken;
    }

    /**
     * @inheritDoc
     */
    public function moveToNextToken(): bool
    {
        $curToken = "";
        $this->currentPosition = $this->nextPosition;
        while (mb_strlen($curChar = $this->getNextChar($this->nextPosition)) > 0) {
            $this->nextPosition++;
            switch ($curChar) {
                // TODO: 注释跳过
                case " ":
                    // 如果开始的时候就有空白，跳过它
                    if (empty($curToken)) {
                        $this->moveCursorToNextChar();
                        continue 2;
                    }
                    // 否则就结束（已经匹配完成）
                    break 2;
                case "\r":
                    if ($this->getNextChar($this->nextPosition) === "\n") {
                        // CRLF换行
                        if (empty($curToken)) {
                            $this->moveCursorToNextChar();
                        }
                        $this->nextPosition++;
                    }
                    // CR换行
                    if (empty($curToken)) {
                        $this->moveCursorToNextLine();
                        continue 2;
                    } else {
                        break 2;
                    }
                case "\n":
                    // LF换行
                    if (empty($curToken)) {
                        $this->moveCursorToNextLine();
                        continue 2;
                    } else {
                        break 2;
                    }
                default:
                    $curToken .= $curChar;
            }
        }
        $this->currentToken = $curToken;
        return $curChar !== "";
    }

    /**
     * @inheritDoc
     */
    public function skipCurrentLine(): bool
    {
        $curPos = $this->currentPosition;
        while ($curChar = $this->getNextChar($curPos)) {
            $curPos++;
            switch ($curChar) {
                case "\r":
                    if ($this->getNextChar($curPos) === "\n") {
                        // CRLF换行
                        $curPos++;
                    }
                    break 2;
                case "\n":
                    break 2;
            }
        }
        $this->nextPosition = $curPos;
        $this->moveCursorToNextLine($curPos - $this->currentPosition);
        return $this->moveToNextToken();
    }

    /**
     * @inheritDoc
     */
    public function skipUntil(string $end = "*/"): bool
    {
        // TODO: Implement skipUntil() method.
        return true;
    }
}